]> git.tdb.fi Git - ext/subsurface.git/blobdiff - dive.h
Separate out the UI from the program logic
[ext/subsurface.git] / dive.h
diff --git a/dive.h b/dive.h
index 15a357f8998ec32bc29ff02351d65c7712507403..1927da7215a68e69ece2ce062cb58e61b6914bd9 100644 (file)
--- a/dive.h
+++ b/dive.h
@@ -1,6 +1,11 @@
 #ifndef DIVE_H
 #define DIVE_H
 
+#include <stdlib.h>
+#include <time.h>
+
+#include <glib.h>
+
 /*
  * Some silly typedefs to make our units very explicit.
  *
@@ -11,7 +16,7 @@
  *
  * We also strive to make '0' a meaningless number saying "not
  * initialized", since many values are things that may not have
* been reported (eg tank pressure or temperature from dive
  * computers that don't support them). But sometimes -1 is an even
  * more explicit way of saying "not there".
  *
@@ -19,7 +24,7 @@
  * temperatures. Doing temperatures in celsius or fahrenheit would
  * make for loss of precision when converting from one to the other,
  * and using millikelvin is SI-like but also means that a temperature
* of '0' is clearly just a missing temperature or tank pressure.
  *
  * Also strive to use units that can not possibly be mistaken for a
  * valid value in a "normal" system without conversion. If the max
@@ -64,26 +69,62 @@ typedef struct {
        int grams;
 } weight_t;
 
-typedef struct {
+struct gasmix {
        fraction_t o2;
        fraction_t he;
-} gasmix_t;
+};
 
 typedef struct {
        volume_t size;
-       pressure_t pressure;
-} tank_type_t;
+       pressure_t workingpressure;
+       const char *description;        /* "LP85", "AL72", "AL80", "HP100+" or whatever */
+} cylinder_type_t;
+
+typedef struct {
+       cylinder_type_t type;
+       struct gasmix gasmix;
+       pressure_t start, end;
+} cylinder_t;
+
+static inline double mm_to_feet(int mm)
+{
+       return mm * 0.00328084;
+}
 
 static inline int to_feet(depth_t depth)
 {
-       return depth.mm * 0.00328084 + 0.5;
+       return mm_to_feet(depth.mm) + 0.5;
+}
+
+static double mkelvin_to_C(int mkelvin)
+{
+       return (mkelvin - 273150) / 1000.0;
+}
+
+static double mkelvin_to_F(int mkelvin)
+{
+       return mkelvin * 9 / 5000.0 - 459.670;
 }
 
 static inline int to_C(temperature_t temp)
 {
        if (!temp.mkelvin)
                return 0;
-       return (temp.mkelvin - 273150) / 1000;
+       return mkelvin_to_C(temp.mkelvin) + 0.5;
+}
+
+static inline int to_F(temperature_t temp)
+{
+       if (!temp.mkelvin)
+               return 0;
+       return mkelvin_to_F(temp.mkelvin) + 0.5;
+}
+
+static inline int to_K(temperature_t temp)
+{
+       if (!temp.mkelvin)
+               return 0;
+       return (temp.mkelvin + 499)/1000;
 }
 
 static inline int to_PSI(pressure_t pressure)
@@ -95,25 +136,44 @@ struct sample {
        duration_t time;
        depth_t depth;
        temperature_t temperature;
-       pressure_t tankpressure;
-       int tankindex;
+       pressure_t cylinderpressure;
+       int cylinderindex;
 };
 
-#define MAX_MIXES (4)
+#define MAX_CYLINDERS (8)
 
 struct dive {
-       const char *name;
+       int number;
        time_t when;
+       char *location;
+       char *notes;
+       char *divemaster, *buddy;
+       double latitude, longitude;
        depth_t maxdepth, meandepth;
        duration_t duration, surfacetime;
        depth_t visibility;
        temperature_t airtemp, watertemp;
-       pressure_t beginning_pressure, end_pressure;
-       gasmix_t gasmix[MAX_MIXES];
-       int samples;
+       cylinder_t cylinder[MAX_CYLINDERS];
+       int samples, alloc_samples;
        struct sample sample[];
 };
 
+/*
+ * We keep our internal data in well-specified units, but
+ * the input and output may come in some random format. This
+ * keeps track of those units.
+ */
+struct units {
+       enum { METERS, FEET } length;
+       enum { LITER, CUFT } volume;
+       enum { BAR, PSI, PASCAL } pressure;
+       enum { CELSIUS, FAHRENHEIT, KELVIN } temperature;
+       enum { KG, LBS } weight;
+};
+
+extern const struct units SI_units, IMPERIAL_units;
+extern struct units input_units, output_units;
+
 extern int verbose;
 
 struct dive_table {
@@ -123,6 +183,9 @@ struct dive_table {
 
 extern struct dive_table dive_table;
 
+extern int selected_dive;
+#define current_dive (get_dive(selected_dive))
+
 static inline struct dive *get_dive(unsigned int nr)
 {
        if (nr >= dive_table.nr)
@@ -131,8 +194,52 @@ static inline struct dive *get_dive(unsigned int nr)
 }
 
 extern void parse_xml_init(void);
-extern void parse_xml_file(const char *filename);
+extern void parse_xml_file(const char *filename, GError **error);
+
+extern void show_dive_info(struct dive *);
+extern void flush_dive_info_changes(struct dive *);
+
+extern void show_dive_equipment(struct dive *);
+extern void flush_dive_equipment_changes(struct dive *);
+
+extern void update_dive(struct dive *new_dive);
+extern void save_dives(const char *filename);
+
+static inline unsigned int dive_size(int samples)
+{
+       return sizeof(struct dive) + samples*sizeof(struct sample);
+}
+
+extern time_t utc_mktime(struct tm *tm);
+
+extern struct dive *alloc_dive(void);
+extern void record_dive(struct dive *dive);
+
+extern struct sample *prepare_sample(struct dive **divep);
+extern void finish_sample(struct dive *dive, struct sample *sample);
+
+extern void report_dives(void);
+extern struct dive *fixup_dive(struct dive *dive);
+extern struct dive *try_to_merge(struct dive *a, struct dive *b);
+
+extern void renumber_dives(int nr);
+
+/* UI related protopypes */
+
+extern void init_ui(int argc, char **argv);
+
+extern void run_ui(void);
+
+extern void report_error(GError* error);
+
+extern void dive_list_update_dives(void);
+extern void flush_divelist(struct dive *dive);
+
+extern int open_import_file_dialog(char *filterpattern, char *filtertext, 
+                               void(* parse_function)(char *));
+#define DIVE_ERROR_PARSE 1
 
-void save_dives(const char *filename);
+const char *weekday(int wday);
+const char *monthname(int mon);
 
 #endif /* DIVE_H */
@@ -11,7 +16,7 @@
  * We also strive to make '0' a meaningless number saying "not
  * initialized", since many values are things that may not have
- * been reported (eg tank pressure or temperature from dive
+ * been reported (eg cylinder pressure or temperature from dive
  * computers that don't support them). But sometimes -1 is an even
  * more explicit way of saying "not there".
@@ -19,7 +24,7 @@
  * make for loss of precision when converting from one to the other,
  * and using millikelvin is SI-like but also means that a temperature
- * of '0' is clearly just a missing temperature or tank pressure.
+ * of '0' is clearly just a missing temperature or cylinder pressure.
  *
  * Also strive to use units that can not possibly be mistaken for a